/*
 *
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <linux/fs.h>
#include <linux/jfs/jfs_incore.h>
#include <linux/jfs/jfs_lock.h>
#include <linux/jfs/jfs_filsys.h>
#include <linux/jfs/jfs_debug.h>
#include <linux/locks.h>

extern int jfs_commit_inode(struct inode *, int);
extern int jfs_ftruncate(struct inode *, loff_t);

int jfs_fsync(struct file *file, struct dentry *dentry, int datasync)
{
	struct inode *inode = dentry->d_inode;
	int rc = 0;

	rc = fsync_inode_buffers(inode);
	if (datasync || !(inode->i_state & I_DIRTY))
		return rc;

	IWRITE_LOCK(inode);
	rc = jfs_commit_inode(inode, 1);
	IWRITE_UNLOCK(inode);
	return rc;
}

struct file_operations jfs_file_operations = {
	write:		generic_file_write,
	read:		generic_file_read,
	mmap:		generic_file_mmap,
	fsync:		jfs_fsync,
};

static void jfs_truncate(struct inode *ip)
{
	jFYI(1, ("jfs_truncate: size = 0x%lx\n", (ulong) ip->i_size));

	(void) jfs_ftruncate(ip, ip->i_size);
}

struct inode_operations jfs_file_inode_operations = {
	truncate:	jfs_truncate,
};
